% Markov Switching Bayesian Structural FAVAR  by FANG ZHANG, June 2014
% -------------------------------------------------------------------------
% The model is:
% 
%   FACTOR (OBSERVATION) EQUATION:
%                     _      _
%                    |  F(t)  |
%      X(t)  =  L  x |        | + e(t) 
%                    |_ Y(t) _|
% 
% 
%   VAR (STATE) EQUATION:
%     _      _            _        _
%    |  F(t)  |          |  F(t-1)  |
%    |        |  = PHI x |          |  +  u(t) 
%    |_ Y(t) _|          |_ Y(t-1) _|
% 
% 
%  with u(t)~N(0,S_F), with S_F = diag(diag(S_F))
 
%  Error EQUATION:
%     _      _            _        _       _      _
%    |  e1(t)  |          |  e1(t-1) |    |  v1(t) |
%    |    :    |  = RHO x |    :     |  + |    :   |
%    |_ eN(t) _|          |_ eN(t-1) |    |  vN(t) |
% 
% 
%  with v(t)~N(0,SIGMA_v), with SIGMA_v = diag(diag((sigma_1),...., (sigma_N))).

% -------------------------------------------------------------------------

clear all;
close all;

clc;
randn('state',sum(100*clock)); 
rand('twister',sum(100*clock));

%============================================================================
%-------------------- Data  ------------------------------
%=============================================================================
  
%== load data 
load ('FFR.mat','FFR');
load PCE;
xdataraw=PCE(205:end,:);   
FFR=FFR(205:end,:); 

year=1976:1/12:2014.25;
Ttrun=0; 

year=year(2:end)'; % datam starts in 1959:1 and ends in 2014:3
year=year(Ttrun+1:end);

T=size(year,1); 
N=size(xdataraw,2); 
M=size(FFR,2); 

xdata=zeros(T,N);
for i_x = 1:N 
    xdata(:,i_x) = transx(xdataraw(Ttrun+1:end,i_x),5); 
end

X = xdata-repmat(mean(xdata,1),T,1); 
X = X./repmat(std(X,0,1),T,1);

ydata=FFR(Ttrun+1:end,:)-mean(FFR(Ttrun+1:end,:));
stdffr = std(FFR(Ttrun+1:end,:));
Y = ydata;
Y=Y./stdffr;

S1=1:1:150; S0=101:1:T-2;
S0=S0';S1=S1';

%============================================================================
%----------------------------PRELIMINARIES---------------------------------
%=============================================================================

nrep = 10000;  
nburn = 6000;   
nsave = 10000; 

nthin = 1;   
it_print = 20;  

K = 2;               
p = K+M;             
plag = 2;            

[F0,Lf,e]=tmp(X,0,K,0);  

XY=X;
FY=[F0,Y];  

L = (olss(XY,FY))';

xi= X - Y*L(:,end)'; 
for ii=1:40;
    [F0,Lf0,e0] = tmp(xi,0,K,0);
    FY0=[F0,Y];
    LL=olssvd(X,FY0)';
    xi = X - Y*LL(:,end)'; 
    
end
FY=[F0,Y]; 
LS0 = (olss(XY(S0,:),FY(S0,:)))'; 
LS1 = (olss(XY(S1,:),FY(S1,:)))'; 

e(S0,:) = XY(S0,:) - FY(S0,:)*LS0';
e(S1,:) = XY(S1,:) - FY(S1,:)*LS1';

FY_lag = mlag2(FY,plag);
FY_lag = FY_lag(plag+1:T,:);
FY_temp = FY(plag+1:T,:);

e_lag = mlag2(e,plag);
e_lag = e_lag(plag+1:T,:);
e_temp = e(plag+1:T,:);

X_lag = mlag2(X,plag);
X_lag = X_lag(plag+1:T,:);
X_temp = X(plag+1:T,:);

PHIS0 = inv(FY_lag(S0,:)'*FY_lag(S0,:))*(FY_lag(S0,:)'*FY_temp(S0,:));   
PHIS1 = inv(FY_lag(S1,:)'*FY_lag(S1,:))*(FY_lag(S1,:)'*FY_temp(S1,:)); 
SSE_FS0 = (FY_temp(S0,:) - FY_lag(S0,:)*PHIS0)'*(FY_temp(S0,:) - FY_lag(S0,:)*PHIS0); 
SSE_FS1 = (FY_temp(S1,:) - FY_lag(S1,:)*PHIS1)'*(FY_temp(S1,:) - FY_lag(S1,:)*PHIS1); 
S_F_S0 = SSE_FS0./(size(S0,1)-p); S_F_S1 = SSE_FS1./(size(S0,1)-p); 
S_F_S0 = diag(diag(S_F_S0)); S_F_S1 = diag(diag(S_F_S1));

rhoS0=zeros(N,plag); 
rhoS1=zeros(N,plag);
for i=1:N
    rhoS0(i,:)=(inv([e_lag(S0,i) e_lag(S0,N+i)]'*[e_lag(S0,i) e_lag(S0,N+i)])*([e_lag(S0,i) e_lag(S0,N+i)]'*e_temp(S0,i)))'; 
    rhoS1(i,:)=(inv([e_lag(S1,i) e_lag(S1,N+i)]'*[e_lag(S1,i) e_lag(S1,N+i)])*([e_lag(S1,i) e_lag(S1,N+i)]'*e_temp(S1,i)))';  
end
RHOS0=[diag(rhoS0(:,1));diag(rhoS0(:,2))];
RHOS1=[diag(rhoS1(:,1));diag(rhoS1(:,2))];

SSE_eS0 = (e_temp(S0,:) - e_lag(S0,:)*RHOS0)'*(e_temp(S0,:) - e_lag(S0,:)*RHOS0);% v*v'
S_eS0 = SSE_eS0./(size(S0,1)-p);
SIGMA_vS0 = diag(diag(S_eS0));

SSE_eS1 = (e_temp(S1,:) - e_lag(S1,:)*RHOS1)'*(e_temp(S1,:) - e_lag(S1,:)*RHOS1);% v*v'
S_eS1 = SSE_eS0./(size(S1,1)-p);
SIGMA_vS1 = diag(diag(S_eS1));

PHIS0_mat = [PHIS0' ; eye(p*(plag-1)) zeros(p*(plag-1),p)];
PHIS1_mat = [PHIS1' ; eye(p*(plag-1)) zeros(p*(plag-1),p)];
RHOS0_mat = [RHOS0' ; eye(N*(plag-1)) zeros(N*(plag-1),N)];
RHOS1_mat = [RHOS1' ; eye(N*(plag-1)) zeros(N*(plag-1),N)];

LiS0_pr=LS0;
LiS1_pr=LS1;
Li_prvar = 4*eye(p);

PHIS0_pr = PHIS0;   
phiS0_pr = PHIS0_pr(:); 

PHIS1_pr = PHIS1;  
phiS1_pr = PHIS1_pr(:); 

phi_V_prior = 10*eye(size(p*plag*p,1)); 

S_F_S0pr=S_F_S0;
S_F_S1pr=S_F_S1;

phi_vpr = p;             
phi_Spr = eye(p);          
phi_invSpr = inv(phi_Spr);
    
rhoi_V_prior = 10*eye(plag); 
rhoiS0_pr=zeros(N,plag);
rhoiS1_pr=zeros(N,plag);
for i=1:N
rhoiS0_pr(i,:) = rhoS0(i,:);   
rhoiS1_pr(i,:) = rhoS1(i,:);   
end

SIGMA_vS0pr=SIGMA_vS0;
SIGMA_vS1pr=SIGMA_vS1;

rhoi_vpr = N;            
rhoi_Spr = eye(N);
rhoi_invSpr = inv(rhoi_Spr);

nhor = 60;

bigj = zeros(p,p*plag);
bigj(1:p,1:p) = eye(p);
ebigj = zeros(N,N*plag);
ebigj(1:N, 1:N)=eye(N);

p00=0.8;p11=0.8;

U1_01=2;U1_00=8;
U1_10=1;U1_11=9;
    
SMM=zeros(T-plag,nsave);
p00MM=zeros(1,nsave); p11MM=zeros(1,nsave);
LS0MM=zeros(N,p,nsave);
LS1MM=zeros(N,p,nsave);
rhoS0MM=zeros(N,plag,nsave);
rhoS1MM=zeros(N,plag,nsave);
PHIS0MM=zeros(plag*p,p,nsave);
PHIS1MM=zeros(plag*p,p,nsave);
s_f_S0MM=zeros(p,nsave);
s_f_S1MM=zeros(p,nsave);

sigma_vS0MM=zeros(N,nsave);
sigma_vS1MM=zeros(N,nsave);

flt_prMM=zeros(2,T-plag,nsave);

k=1;
fileresult='resultkh.mat';

%=============================================================================================
%====================================== START SAMPLING ========================================
%==============================================================================================
tic; 
disp('Number of iterations');

for irep = 1:nrep + nburn    %
    
    if mod(irep,it_print) == 0
        disp(irep); toc;
    end
    
    % -----------------------------------------------------------------------------------------
    %   STEP I: Sample States and Transition Probablity
    % -----------------------------------------------------------------------------------------

    [flt_pr,S]=kimnelson(p00,p11,LS0,LS1,PHIS0,PHIS1,S_F_S0,S_F_S1,RHOS0,RHOS1,SIGMA_vS0,SIGMA_vS1,X_temp,FY_temp,FY_lag);

    SSum=sum(S); 
    SSum1=size(S,1); 

    S0=zeros(SSum1-SSum,1); 
    S1=zeros(SSum,1); 
    i0=1;
    i1=1;
    for i=1:1:SSum1
       if S(i,1)==0
           S0(i0,1)=i;
           i0=i0+1;
       elseif S(i,1)==1
           S1(i1,1)=i;
           i1=i1+1;
       end
    end
    
    tranmat=trancount(S+1,[1;2]);
    a1tt=betarnd(tranmat(1,2)+U1_01,tranmat(1,1)+U1_00);
    b1tt=betarnd(tranmat(2,1)+U1_10,tranmat(2,2)+U1_11);
    p00=1-a1tt;
    p11=1-b1tt; 
    
    % -----------------------------------------------------------------------------------------
    %   STEP II: Sample L
    % ----------------------------------------------------------------------------------------- 

    Xhat(S0,:)=X_temp(S0,:)-X_lag(S0,:)*RHOS0; 
    Xhat(S1,:)=X_temp(S1,:)-X_lag(S1,:)*RHOS1; 
    
    FYhat1S0=repmat(FY_temp(S0,1),1,N)-[repmat(FY_lag(S0,1),[1,N]),repmat(FY_lag(S0,p+1),1,N)]*RHOS0; 
    FYhat2S0=repmat(FY_temp(S0,2),1,N)-[repmat(FY_lag(S0,2),[1,N]),repmat(FY_lag(S0,p+2),1,N)]*RHOS0;    
    FYhat3S0=repmat(FY_temp(S0,3),1,N)-[repmat(FY_lag(S0,3),[1,N]),repmat(FY_lag(S0,p+3),1,N)]*RHOS0;
    
    FYhat1S1=repmat(FY_temp(S1,1),1,N)-[repmat(FY_lag(S1,1),[1,N]),repmat(FY_lag(S1,p+1),1,N)]*RHOS1; 
    FYhat2S1=repmat(FY_temp(S1,2),1,N)-[repmat(FY_lag(S1,2),[1,N]),repmat(FY_lag(S1,p+2),1,N)]*RHOS1;    
    FYhat3S1=repmat(FY_temp(S1,3),1,N)-[repmat(FY_lag(S1,3),[1,N]),repmat(FY_lag(S1,p+3),1,N)]*RHOS1;
    
    XhatS0=Xhat(S0,:); XhatS1=Xhat(S1,:); 
    
    eS0=zeros(T,N); eS1=zeros(T,N);
    for i=1:N
                  
            FYhatS0=[FYhat1S0(:,i) FYhat2S0(:,i) FYhat3S0(:,i)]; 
            Li_postvarS0 = inv(inv(Li_prvar) + inv(SIGMA_vS0pr(i,i))*FYhatS0'*FYhatS0); 
            Li_postmeanS0 = Li_postvarS0*(inv(Li_prvar)*LiS0_pr(i,:)'+inv(SIGMA_vS0pr(i,i))*FYhatS0'*XhatS0(:,i));
            LidrawS0 = Li_postmeanS0' + randn(1,p)*Li_postvarS0;
            LS0(i,:) = LidrawS0;
        
            FYhatS1=[FYhat1S1(:,i) FYhat2S1(:,i) FYhat3S1(:,i)];
            Li_postvarS1 = inv(inv(Li_prvar) + inv(SIGMA_vS1pr(i,i))*FYhatS1'*FYhatS1); 
            Li_postmeanS1 = Li_postvarS1*(inv(Li_prvar)*LiS1_pr(i,:)'+inv(SIGMA_vS1pr(i,i))*FYhatS1'*XhatS1(:,i));
            LidrawS1 = Li_postmeanS1' + randn(1,p)*Li_postvarS1;
            LS1(i,:) = LidrawS1;

    end  
    
 % -----------------------------------------------------------------------------------------
%   STEP III: Sample PHI and S_F
% -----------------------------------------------------------------------------------------
    
        phiS0_OLS=PHIS0(:);
        phiS1_OLS=PHIS1(:);
        
        varianceS0=kron(inv(S_F_S0pr),eye(size(S0,1))); 
        varianceS1=kron(inv(S_F_S1pr),eye(size(S1,1)));
        
        ZS0=kron(eye(p),FY_lag(S0,:)); 
        ZS1=kron(eye(p),FY_lag(S1,:)); 
        FY_tempS0=FY_temp(S0,:); FY_tempS1=FY_temp(S1,:);
        ZFYS0=FY_tempS0(:); ZFYS1=FY_tempS1(:);
         
       
        phiS0_V_post = inv(inv(phi_V_prior) + ZS0'*varianceS0*ZS0);
        phiS0_post = phiS0_V_post*(inv(phi_V_prior)*phiS0_pr + ZS0'*varianceS0*ZFYS0); 
        phiS0draw = phiS0_post + chol(phiS0_V_post)'*randn(plag*p*p,1); 
        PHIS0= reshape(phiS0draw,plag*p,p); 
        
        phiS1_V_post = inv(inv(phi_V_prior) + ZS1'*varianceS1*ZS1);
        phiS1_post = phiS1_V_post*(inv(phi_V_prior)*phiS1_pr + ZS1'*varianceS1*ZFYS1);
        phiS1draw = phiS1_post + chol(phiS1_V_post)'*randn(plag*p*p,1); 
        PHIS1= reshape(phiS1draw,plag*p,p);       
        
       
        phiS0_vpost = size(S0,1) + plag + phi_vpr;
        phiS0_Spost = phi_Spr + (FY_temp(S0,:) - FY_lag(S0,:)*PHIS0)'*(FY_temp(S0,:) - FY_lag(S0,:)*PHIS0);
        S_F_S0 = inv(wish(inv(phiS0_Spost),phiS0_vpost));
        S_F_S0=diag(diag(S_F_S0));
                
        phiS1_vpost = size(S1,1) +plag + phi_vpr;
        phiS1_Spost = phi_Spr + (FY_temp(S1,:) - FY_lag(S1,:)*PHIS1)'*(FY_temp(S1,:) - FY_lag(S1,:)*PHIS1);
        S_F_S1 = inv(wish(inv(phiS1_Spost),phiS1_vpost));
        S_F_S1=diag(diag(S_F_S1));
        
  
    PHIS0_mat(1:p,:) = PHIS0'; 
    PHIS1_mat(1:p,:) = PHIS1';
    
    while max(abs(eig(PHIS0_mat)))>0.999
        phiS0_vec = phiS0_pr + chol(phiS0_V_post)'*randn(p*p*plag,1);
        PHIS0 = reshape(phiS0_vec,p*plag,p);   
        PHIS0_mat(1:p,:) = PHIS0';
    end
    
    while max(abs(eig(PHIS1_mat)))>0.999
        phiS1_vec = phiS1_pr + chol(phiS1_V_post)'*randn(p*p*plag,1);
        PHIS1 = reshape(phiS1_vec,p*plag,p);   
        PHIS1_mat(1:p,:) = PHIS1';
    end
    
% -----------------------------------------------------------------------------------------
%   STEP III': Sample RHO and SIGMA_v
% -----------------------------------------------------------------------------------------

    rhoS0_post=rhoiS0_pr; 
    rhoS0_V_post=zeros(N,plag,plag);
    rhoS1_post=rhoiS1_pr; 
    rhoS1_V_post=zeros(N,plag,plag);
    rhoS0=zeros(N,plag);rhoS1=zeros(N,plag);
    for ii=1:1:N
            ZeS0=kron(eye(1),[e_lag(S0,ii) e_lag(S0,N+ii)]); 
            ZeS1=kron(eye(1),[e_lag(S1,ii) e_lag(S1,N+ii)]); 

            variance_eS0=kron(inv(SIGMA_vS0pr(ii,ii)),eye(size(S0,1))); 
            variance_eS1=kron(inv(SIGMA_vS1pr(ii,ii)),eye(size(S1,1))); 

            rhoiS0_V_post=inv(inv(rhoi_V_prior) + ZeS0'*variance_eS0*ZeS0);
            rhoiS1_V_post=inv(inv(rhoi_V_prior) + ZeS1'*variance_eS1*ZeS1);

            rhoS0_post(ii,:)=rhoiS0_V_post*(inv(rhoi_V_prior)*rhoiS0_pr(ii,:)' + ZeS0'*variance_eS0*e_temp(S0,ii));
            rhoS0_V_post(ii,:,:)=rhoiS0_V_post;
            rhoiS0draw = rhoS0_post(ii,:)' + chol(rhoiS0_V_post)'*randn(plag,1); 
  
            rhoS0(ii,:)= rhoiS0draw'; 

            rhoS1_post(ii,:)=rhoiS1_V_post*(inv(rhoi_V_prior)*rhoiS1_pr(ii,:)' + ZeS1'*variance_eS1*e_temp(S1,ii));
            rhoS1_V_post(ii,:,:)=rhoiS1_V_post;
            rhoiS1draw = rhoS1_post(ii,:)' + chol(rhoiS1_V_post)'*randn(plag,1); 
            
            rhoS1(ii,:)= rhoiS1draw'; 
    end
         
    RHOS0=[diag(rhoS0(:,1));diag(rhoS0(:,2))];
    RHOS1=[diag(rhoS1(:,1));diag(rhoS1(:,2))];
    
    RHOS0_mat = [RHOS0' ; eye(N*(plag-1)) zeros(N*(plag-1),N)];
    RHOS1_mat = [RHOS1' ; eye(N*(plag-1)) zeros(N*(plag-1),N)];
    
    rhoiS0_vpost = size(S0,1)+plag + rhoi_vpr; 
    rhoiS1_vpost = size(S1,1)+plag + rhoi_vpr; 
      
    vS0=zeros(size(S0,1),N);
    vS1=zeros(size(S1,1),N);
      for jj=1:N
            vS0(:,jj)= e_temp(S0,jj) - [e_lag(S0,jj),e_lag(S0,N+jj)]*rhoS0(jj,:)';
            vS1(:,jj)= e_temp(S1,jj) - [e_lag(S1,jj),e_lag(S1,N+jj)]*rhoS1(jj,:)';
      end
    rhoiS0_Spost = rhoi_Spr + vS0'*vS0;
    SIGMA_vS0draw= inv(wish(inv(rhoiS0_Spost),rhoiS0_vpost));
    SIGMA_vS0=diag(diag(SIGMA_vS0draw));  
        
    rhoiS1_Spost = rhoi_Spr + vS1'*vS1;
    SIGMA_vS1draw= inv(wish(inv(rhoiS1_Spost),rhoiS1_vpost));
    SIGMA_vS1=diag(diag(SIGMA_vS1draw));         
    
% -----------------------------------------------------------------------------------------
%   Save and Compute Results
% --------------------------------------------------------------------------------

    if irep  > nburn
        if irep-nburn < k*nsave+1
            SMM(:,irep-nburn-(k-1)*nsave)=S;
            p00MM(:,irep-nburn-(k-1)*nsave)=p00; 
            p11MM(:,irep-nburn-(k-1)*nsave)=p11;
            LS0MM(:,:,irep-nburn-(k-1)*nsave)=LS0;
            LS1MM(:,:,irep-nburn-(k-1)*nsave)=LS1;
            rhoS0MM(:,:,irep-nburn-(k-1)*nsave)=rhoS0;  
            rhoS1MM(:,:,irep-nburn-(k-1)*nsave)=rhoS1;
            PHIS0MM(:,:,irep-nburn-(k-1)*nsave)=PHIS0;
            PHIS1MM(:,:,irep-nburn-(k-1)*nsave)=PHIS1;
            s_f_S0MM(:,irep-nburn-(k-1)*nsave)=diag(S_F_S0);
            s_f_S1MM(:,irep-nburn-(k-1)*nsave)=diag(S_F_S1);
            sigma_vS0MM(:,irep-nburn-(k-1)*nsave)=diag(SIGMA_vS0);
            sigma_vS1MM(:,irep-nburn-(k-1)*nsave)=diag(SIGMA_vS1);
            flt_prMM(:,:,irep-nburn-(k-1)*nsave)=flt_pr;
        else if k<10
                fileresult(7)=num2str(k);
                save(fileresult,'SMM','p00MM','p11MM','LS0MM','LS1MM','PHIS0MM','PHIS1MM','rhoS0MM','rhoS1MM','s_f_S0MM','s_f_S1MM','sigma_vS0MM','sigma_vS1MM','flt_prMM');
                k=k+1;
            else 
                fileresult(7:8)=num2str(k);
                save(fileresult,'SMM','p00MM','p11MM','LS0MM','LS1MM','PHIS0MM','PHIS1MM','rhoS0MM','rhoS1MM','s_f_S0MM','s_f_S1MM','sigma_vS0MM','sigma_vS1MM','flt_prMM');
                k=k+1;
            end
        end
        
    end

end p 

 fileresult(7:8)=num2str(k);
 save(fileresult,'SMM','p00MM','p11MM','LS0MM','LS1MM','PHIS0MM','PHIS1MM','rhoS0MM','rhoS1MM','s_f_S0MM','s_f_S1MM','sigma_vS0MM','sigma_vS1MM','flt_prMM');

save('result_PCE.mat');

clc;
toc; 
